#                             AlloppDT.

# Started 2012-08-04. 
# This is for the network model with an arbitrary number
# of diploids (>=2), tetraploids (>=1) and hybridizations. The BEAST XML has
# changed significantly from the one-hybridization case.
# The MUL-tree model is omitted from this code. The changes are enough to 
# have a new name AlloppDT



library(ape)

################################################################################
###################### Setting up file paths, loading code #####################

cat("AlloppDT started\n")
Args <- commandArgs(TRUE)
Args <- c("VOSTROWIN7", "I:/2016-03-25-AlloppNET-output/E")
stopifnot(length(Args) == 2) 

setwd("C:/Users/Work/AAA/Programming/Goteborg/AlloppRcode-nhybs")

# Sets variables computer, config.dir, java.classpath, seqgenbasecmd
source("AlloppDT_0filepaths.r")

# Code providing functions
source("AlloppDT_0randomseeds.r")
source("AlloppDT_1readconfig.r")
source("AlloppDT_2makemultree.r")
source("AlloppDT_3makegtrees.r")
source("AlloppDT_4makeseqfiles.r")
source("AlloppDT_5beastxml_toplevel.r")
source("AlloppDT_6beastxml_bits.r")
source("AlloppDT_7utils.r")

################################################################################
######################### Top level functions ##################################


make.genetrees.for.one.GNT.one.replicate <- function(config, i, r) { 
  # Simulate the gene trees
  for (g in 1:config$configG[i]) {
    gt <- make.one.genetree.as.Newick(multree, config$configT[i], getrandomseed(r, g))
    sink(fpath.gtree(config, i, r, g))
    cat(gt)
    sink(NULL)
    pdf(file=fpath.gtree.pdf(config, i, r, g))
    phylo.tree <- read.tree(text=gt)
    plot.phylo(phylo.tree, edge.color="grey")
    axisPhylo()
    title(paste("Simulated gene tree: rep", r, "gene", g))
    dev.off()
    }
  }




make.sequences.for.one.GNT.one.replicate <- function(config, i, r) { 
  # Simulate the sequence alignments
  for (g in 1:config$configG[i]) {  
    SeqGen.cmd <- paste(seqgenbasecmd, 
                        " -l", config$seqs$len, 
                        " -n1 -z", getrandomseed(r, g), 
                        " ", fpath.gtree(config, i, r, g), 
                        " -y ", fpath.seq(config, i, r, g), 
                        sep="")
    cat("Running seqgen with command:", SeqGen.cmd, "\n")
    system(SeqGen.cmd)
    }
  }
  

make.beastxml.for.one.GNT.one.replicate <- function(config, i, r) {  
   make.genetrees.for.one.GNT.one.replicate(config, i, r)
   make.sequences.for.one.GNT.one.replicate(config, i, r) 

  # Make BEAST XML
  sink(fpath.beastxml(config, i, r))
  make.beastxml.from.sim.network(config$network, config$configG[i],
                collect.taxa(multree),
                popmean.eventrate.relativerates.prior=c(-9,1, 4.6,2, 0,1),
                fpath.seqs.fpathbase(config, i, r), 
                fpath.sampledgtrees.fpathbase(config, i, r),
                fpath.sampledmultrees(config, i, r), 
                fpath.sampledparams(config, i, r), 
                fpath.DBUGTUNE(config, i, r))
  sink(NULL)
  }





run.beast.for.one.GNT.one.replicate <- function(config, i, r) {
  if (beast.randomseed > 0) {
    randseed <- beast.randomseed
    } else {
    randseed <- 1000*i + r
    }
  BEAST.cmd <- paste("java -enableassertions -classpath", java.classpath, 
          "dr.app.beast.BeastMain -seed", randseed, "-overwrite -working", 
          fpath.beastxml(config, i, r))
  cat(BEAST.cmd, "\n")
  errorcode <- system(BEAST.cmd)
  stopifnot(errorcode == 0)  # don't seem to work
  }
  
  
  
  
run.treeannotator.for.one.GNT.one.replicate <- function(config, i, r, stree.burnin, gtree.burnin) {
#   for (g in 1:config$configG[i]) {
#     TreeAnnotator.cmd <-paste("java -classpath", java.classpath, 
#           "dr.app.tools.TreeAnnotator -burnin", gtree.burnin, 
#           fpath.sampledgtree(config, i, r, g),
#           fpath.mccgtree(config, i, r, g)
#           )
#     system(TreeAnnotator.cmd)
#     }
  TreeAnnotator.cmd <-paste("java -classpath", java.classpath, 
          "dr.app.tools.TreeAnnotator -burnin", stree.burnin,
          fpath.sampledmultrees(config, i, r),
          fpath.mccmultree(config, i, r)
          )      
   system(TreeAnnotator.cmd)
  }  
  

convert.concensustrees.to.pdf <- function(config, i, r) {
  for (g in 1:config$configG[i]) {
    pdf(file=fpath.mccgtree.pdf(config, i, r, g))
    plot.tree.from.nexus.file(fpath.mccgtree(config, i, r, g), branchcolour="brown")
    title(paste("Concensus gene tree: rep", r, "gene", g))
    dev.off()
    }
  pdf(file=fpath.mccmultree.pdf(config, i, r))
  plot.tree.from.nexus.file(fpath.mccmultree(config, i, r), branchcolour="red")
  title(paste("Concensus MUL-tree: rep", r))
  dev.off()    
  }  
    
    
    
  
################################################################################
########################## Business starts here ################################


cat("AlloppDT running with directory", config.dir, "\n")

config <- read.config(config.dir)

cat("Config has beeen read\n")

nofGNT <- length(config$configG)
beast.randomseed <- config$progparams$beastseed
beast.chain.length <- config$progparams$beastchainlength
beast.screen.logevery <- config$progparams$beastscreen.logevery
beast.params.logevery <- config$progparams$beastparams.logevery
beast.gtrees.logevery <- config$progparams$beastgtrees.logevery
beast.multree.logevery <- config$progparams$beastmultree.logevery
beast.dbugtune.logevery <- config$progparams$beastdbugtune.logevery




replicate.sets <- NULL
for (i in 1:nofGNT) {
  R <- config$configR[i]
  replicate.sets <- c(replicate.sets, list(1:R))
  }


# 
# for (i in 1:nofGNT) {
#   
#   dir.create(fpath.GNTdpath(config, i))
#   
#   # Make the MUL-tree
#   multree <- make.multree(config$network)
#   cat("GNT loop: i =", i, "made MUL-tree\n")
#   # add nlins at tips
#   for (nn in 1:length(multree)) {
#     if (multree[[nn]]$nheight == 0) {
#        multree[[nn]]$nlins <- config$configN[i]
#       }
#     }  
#   Newick.multree <- multree.to.Newick(multree)
#   print(Newick.multree)
#   phylo.multree <- read.tree(text=Newick.multree)
#   pdf(file=fpath.multree.pdf(config, i))
#   plot.phylo(phylo.multree)
#   axisPhylo()
#   #hgt <- max(branching.times(phylo.multree)) 
#   #text(c(0,hgt),c(1,2),c("0,1","hgt,2"))
#   #nodelabels(1:11, 1:11)
#   title("Simulated MUL-tree")
#   dev.off()
# 
#   
#   sink(fpath.multree(config, i))
#   print.multree(multree)
#   sink(NULL)
#   
#   for (r in replicate.sets[[i]]) {
#     make.beastxml.for.one.GNT.one.replicate(config, i, r)
#     }
#   }
# 
# 
# 
# for (i in 1:nofGNT) {
#   for (r in replicate.sets[[i]]) {
#     run.beast.for.one.GNT.one.replicate(config, i, r)
#     }
#   }


for (i in 1:nofGNT) {
  for (r in replicate.sets[[i]]) {
    if (config$configG[i]==9 && config$configN[i]==9) {
      run.treeannotator.for.one.GNT.one.replicate(config, i, r, 2001, 201)
    } else {
      run.treeannotator.for.one.GNT.one.replicate(config, i, r, 4001, 401)
    }
  }
}
   
 
# for (i in 1:nofGNT) {
#   for (r in replicate.sets[[i]]) {
#     convert.concensustrees.to.pdf(config, i, r)
#     }
#   }
# 
# 

